import logging
import random

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.template import Context, Library
from django.template.loader import get_template

from elasticsearch import TransportError

from bulbs.content.models import Content
from bulbs.promotion.models import PZone


logger = logging.getLogger(__name__)
register = Library()


@register.simple_tag(takes_context=True)
def end_of_article_video(context):
    '''
    templatetag that renders the end of article video recirc.
    '''
    inactive = context.get("partial", False)
    if inactive:
        return ""

    targeting = context.get("targeting", dict())
    if targeting.get("dfp_specialcoverage", None):
        return ""

    try:
        queryset = PZone.objects.applied(name='end-of-article-videos')
    except ObjectDoesNotExist:
        queryset = None

    if not queryset:
        queryset = Content.search_objects.videos()

    try:
        queryset = queryset[:10]
        video = random.choice(queryset)
    except (IndexError, TransportError):
        logger.error('''Inline video recirc unable to retrieve video.''')
        return ""

    site_name = getattr(settings, "SITE_DISPLAY_NAME", None)
    recirc_text = "Watch Video"
    if site_name:
        recirc_text += " From " + site_name

    base_url = getattr(settings, "VIDEOHUB_BASE_URL", None)
    if base_url is None:
        return ""

    video_src = "{0}/video/{1}.json".format(base_url, video.videohub_ref.id)

    return get_template(
        "recirc/end_of_article.html"
    ).render(
        Context({
            'recirc_text': recirc_text.upper(),
            'video_src': video_src
        })
    )



from django.conf import settings
from django.db import models

from bulbs.content.models import Content
from bulbs.liveblog.utils import get_liveblog_author_model_name

from .tasks import (firebase_delete_entry,
                    firebase_delete_liveblog,
                    firebase_update_entry)


LIVEBLOG_AUTHOR_MODEL_NAME = get_liveblog_author_model_name()


class AbstractLiveBlog(models.Model):
    """Base mixin inherited by each property's concrete LiveBlog implementation (along with base
    content class).
    """

    pinned_content = models.ForeignKey(Content, blank=True, null=True,
                                       related_name='liveblog_pinned')
    recirc_content = models.ManyToManyField(Content, related_name='liveblog_recirc')

    class Meta:
        abstract = True

    def delete(self, *args, **kwargs):
        firebase_delete_liveblog.delay(liveblog_id=self.id)
        return super(AbstractLiveBlog, self).delete(*args, **kwargs)

    def get_absolute_url(self):
        return '/liveblog/{}-{}'.format(self.slug, self.pk)


class LiveBlogEntry(models.Model):

    liveblog = models.ForeignKey(settings.LIVEBLOG_MODEL, related_name='entries')

    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )
    updated_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    published = models.DateTimeField(blank=True, null=True)
    authors = models.ManyToManyField(LIVEBLOG_AUTHOR_MODEL_NAME)
    headline = models.CharField(max_length=255, null=True, blank=True)
    body = models.TextField(blank=True)

    recirc_content = models.ManyToManyField(Content, related_name='liveblog_entry_recirc')

    def save(self, *args, **kwargs):
        super(LiveBlogEntry, self).save(*args, **kwargs)
        firebase_update_entry.delay(liveblog_id=self.liveblog.id,
                                    entry_id=self.id,
                                    published=self.published)

    def delete(self, *args, **kwargs):
        firebase_delete_entry.delay(liveblog_id=self.liveblog.id,
                                    entry_id=self.id)
        return super(LiveBlogEntry, self).delete(*args, **kwargs)

    def get_absolute_url(self):
        return '{}?entry={}'.format(self.liveblog.get_absolute_url(), self.pk)

    class Meta:
        ordering = ['-published']


class LiveBlogResponse(models.Model):

    entry = models.ForeignKey(LiveBlogEntry, related_name='responses')
    ordering = models.IntegerField(blank=True, null=True, default=None)

    internal_name = models.CharField(max_length=255, blank=True, null=True)
    author = models.ForeignKey(LIVEBLOG_AUTHOR_MODEL_NAME, blank=True, null=True)
    body = models.TextField(blank=True)

    class Meta:
        ordering = ['ordering']



from django.conf import settings
from django.db.models.loading import get_model
from django.http import Http404
from django.utils import timezone
from django.views.decorators.cache import cache_control

from bulbs.content.views import BaseContentDetailView

LiveBlogModel = get_model(settings.LIVEBLOG_MODEL)


def entries_page(liveblog, cursor_entry=None):
    entries_query = liveblog.entries.order_by('-published')
    if cursor_entry:
        entries_query = entries_query.filter(
            published__isnull=False,
            published__lte=timezone.now(),
            published__lt=cursor_entry.published)
    else:
        entries_query = entries_query.filter(
            published__lte=timezone.now())
    return entries_query[:10]


class LiveBlogDetailViewMixin(object):

    def get_context_data(self, *args, **kwargs):
        context = super(LiveBlogDetailViewMixin, self).get_context_data(*args, **kwargs)
        context['entries'] = entries_page(liveblog=self.object)
        return context


class LiveBlogMoreEntriesView(BaseContentDetailView):
    model = LiveBlogModel
    template_name = 'liveblog/more_entries.html'
    redirect_correct_path = False

    def get_context_data(self, object):
        context = {}
        cursor_entry = object.entries.filter(
            published__lte=timezone.now,
            pk=self.kwargs['cursor_entry']).first()
        if cursor_entry is None:
            raise Http404()
        context['content'] = self.object
        context['entries'] = entries_page(liveblog=self.object, cursor_entry=cursor_entry)
        return context


class LiveBlogNewEntriesView(BaseContentDetailView):
    model = LiveBlogModel
    template_name = 'liveblog/new_entries.html'
    redirect_correct_path = False

    def get_context_data(self, object):
        context = {}
        if 'entry_ids' not in self.request.GET:
            raise ValueError('param "entry_ids" MUST be specified')
        parsed_entry_ids = [x.strip() for x in self.request.GET['entry_ids'].split(',')]
        context['entries'] = self.object.entries.filter(
            pk__in=parsed_entry_ids,
            published__lte=timezone.now())
        return context

liveblog_more_entries = cache_control(max_age=600)(LiveBlogMoreEntriesView.as_view())
liveblog_new_entries = cache_control(max_age=600)(LiveBlogNewEntriesView.as_view())



from rest_framework import filters


logger = __import__('logging').getLogger(__name__)


class LiveBlogFilterBackend(filters.BaseFilterBackend):
    """
    Optionally include only entries belonging to specified LiveBlog.
    """
    def filter_queryset(self, request, queryset, view):
        liveblog_id = request.GET.get('liveblog')
        if liveblog_id:
            queryset = queryset.filter(liveblog_id=liveblog_id)

        return queryset



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.utils.timezone import utc
from django.conf import settings
import django.db.models.deletion
import datetime


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('liveblog', '0004_auto_20160908_1512'),
    ]

    operations = [
        migrations.AddField(
            model_name='liveblogentry',
            name='created',
            field=models.DateTimeField(default=datetime.datetime(2016, 9, 19, 19, 36, 35, 21936, tzinfo=utc), auto_now_add=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='liveblogentry',
            name='created_by',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, null=True, blank=True),
        ),
        migrations.AddField(
            model_name='liveblogentry',
            name='updated',
            field=models.DateTimeField(default=datetime.datetime(2016, 9, 19, 19, 36, 47, 140826, tzinfo=utc), auto_now=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='liveblogentry',
            name='updated_by',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, null=True, blank=True),
        ),
    ]



from django.apps import apps
from django.conf import settings


def get_liveblog_author_model_name():
    return getattr(settings, "LIVEBLOG_AUTHOR_MODEL", settings.AUTH_USER_MODEL)


def get_liveblog_author_model():
    return apps.get_model(get_liveblog_author_model_name())



from collections import OrderedDict
from elasticsearch_dsl.filter import Not, Type, Term
from rest_framework import viewsets
from rest_framework.pagination import PageNumberPagination
from rest_framework.permissions import AllowAny
from rest_framework.response import Response

from django.template import RequestContext
from django.utils.timezone import now

from bulbs.content.filters import Published
from bulbs.content.models import Content
from bulbs.content.views import ContentListView
from bulbs.special_coverage.models import SpecialCoverage
from bulbs.super_features.utils import get_superfeature_model

from .serializers import GlanceContentSerializer


class RSSView(ContentListView):
    """Really simply, this syndicates Content."""
    template_name = "feeds/rss.xml"
    paginate_by = 20
    feed_title = "RSS Feed"
    utm_params = "utm_medium=RSS&amp;utm_campaign=feeds"

    def get_template_names(self):
        return ["feeds/rss.xml", "feeds/_rss.xml"]

    def get(self, request, *args, **kwargs):
        response = super(RSSView, self).get(request, *args, **kwargs)
        response["Content-Type"] = "application/rss+xml"
        return response

    def get_queryset(self):
        return super(RSSView, self).get_queryset().filter(
            # Exclude all SuperFeatures (until we ever decide to support them)
            Not(filter=Type(value=get_superfeature_model().search_objects.mapping.doc_type))
        ).filter(
            Term(**{'hide_from_rss': False})
        )

    def get_context_data(self, *args, **kwargs):
        context = super(RSSView, self).get_context_data(*args, **kwargs)
        context["full"] = (self.request.GET.get("full", "false").lower() == "true")
        context["images"] = (self.request.GET.get("images", "false").lower() == "true")
        context["build_date"] = now()
        context["title"] = self.feed_title
        context["feed_url"] = self.request.build_absolute_uri()
        context["search_url"] = self.request.build_absolute_uri(
            u"/search?%s" % self.request.META["QUERY_STRING"])

        # OK, so this is kinda brutal. Stay with me here.
        for content in context["page_obj"].object_list:
            feed_path = content.get_absolute_url() + "?" + self.utm_params
            content.feed_url = self.request.build_absolute_uri(feed_path)

        return RequestContext(self.request, context)


class SpecialCoverageRSSView(RSSView):
    """Really simply, this syndicates Content."""
    feed_title = "Special Coverage RSS Feed"

    def get_queryset(self):
        sc_id = self.request.GET.get("special_coverage_id")
        sc_slug = self.request.GET.get("special_coverage_slug")

        if sc_id:
            sc = SpecialCoverage.objects.get(id=sc_id)
        elif sc_slug:
            sc = SpecialCoverage.objects.get(slug=sc_slug)
        else:
            return self.model.objects.none()

        return sc.get_content()[:self.paginate_by]


class GlanceFeedViewSet(viewsets.ReadOnlyModelViewSet):

    model = Content
    serializer_class = GlanceContentSerializer

    queryset = Content.search_objects.search().sort('-last_modified').filter(Published())

    permission_classes = (AllowAny,)

    class GlancePageNumberPagination(PageNumberPagination):
        page_size = 20
        page_size_query_param = 'per_page'
        max_page_size = 100   # Section percolator calls can be expensive

        # Customized for Glance's format
        def get_paginated_response(self, data):
            pagination = OrderedDict([
                ('page', self.page.number),    # TODO: Correct?
                ('per_page', self.page.paginator.per_page),
                ('total', self.page.paginator.count),
                ('content', data)
            ])

            # Per spec, only include 'next' if valid
            if self.get_next_link():
                pagination['next'] = self.get_next_link()

            return Response(pagination)

        def _handle_backwards_compat(self, view):
            # Override parent method to do nothing :)
            # Avoid any deprecated PAGINATION values from property `settings` files.
            # Sites like Onion set 'PAGINATE_BY_PARAM' to 'page_size', which overrrides this class's custom
            # 'per_page' param name.
            # Necessary for DRF v3.1, but won't need this override for DRF v3.3+
            pass

    pagination_class = GlancePageNumberPagination



from celery import shared_task
import requests

from django.conf import settings


# ----------------------------------------------------------------------
# Liveblog Firebase Strategy
#
# Firebase is used only as a messaging channel, to update clients when new entries are available.
#
# The idea is to cache a dictionary of {ENTRY_ID: PULBISH TIME} values for each LiveBlog, so clients
# will know when new content is available. Clients will then request only those Entry IDs that are
# currently published and they don't already have.
#
# Necessary to explicitly list each entry b/c future publish dates may roll over without triggering
# Firebase updates. This way our firebase logic can be dumb and only trigger on save/delete.
# ----------------------------------------------------------------------


def _get_entry_url(liveblog_id, entry_id):
    endpoint = getattr(settings, 'LIVEBLOG_FIREBASE_NOTIFY_ENTRY_ENDPOINT', None)
    if endpoint:
        return endpoint.format(liveblog_id=liveblog_id,
                               entry_id=entry_id)


@shared_task(default_retry_delay=5)
def firebase_update_entry(liveblog_id, entry_id, published):

    url = _get_entry_url(liveblog_id, entry_id)
    if url:
        entry = {'id': entry_id}
        if published:
            entry['published'] = published.isoformat()

        resp = requests.patch(url, json=entry)
        resp.raise_for_status()


@shared_task(default_retry_delay=5)
def firebase_delete_entry(liveblog_id, entry_id):

    url = _get_entry_url(liveblog_id, entry_id)
    if url:

        resp = requests.delete(url)

        resp.raise_for_status()


@shared_task(default_retry_delay=5)
def firebase_delete_liveblog(liveblog_id):
    endpoint = getattr(settings, 'LIVEBLOG_FIREBASE_NOTIFY_ENTRIES_ENDPOINT', None)
    if endpoint:
        url = endpoint.format(liveblog_id=liveblog_id)

        resp = requests.delete(url)

        resp.raise_for_status()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('liveblog', '0003_auto_20160825_1459'),
    ]

    operations = [
        migrations.AlterField(
            model_name='liveblogentry',
            name='headline',
            field=models.CharField(max_length=255, null=True, blank=True),
        ),
        migrations.AlterField(
            model_name='liveblogresponse',
            name='author',
            field=models.ForeignKey(to=settings.AUTH_USER_MODEL, blank=True, null=True),
        ),
    ]



import requests

from django.apps import apps
from django.conf import settings
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    help = "Set a provided FeatureType as available for Facebook Instant Articles."

    def add_arguments(self, parser):
        parser.add_argument(
            "--liveblog",
            help="Rebuild specific liveblog cache",
            type=int
        )
        parser.add_argument(
            '--verbose',
            action='store_true',
            default=False,
            help='Verbose mode')

    def handle(self, *args, **options):

        LiveBlog = apps.get_model(settings.LIVEBLOG_MODEL)

        qs = LiveBlog.objects.all()

        if options['liveblog']:
            qs = qs.filter(pk=options['liveblog'])

        for liveblog in qs.all():

            endpoint = settings.LIVEBLOG_FIREBASE_NOTIFY_ENTRIES_ENDPOINT
            url = endpoint.format(liveblog_id=liveblog.id)

            if liveblog.is_published:

                entries = {}
                for e in liveblog.entries.all():
                    entries[e.id] = {'id': e.id}

                    if e.published:
                        entries[e.id]['published'] = e.published.isoformat()

                if options['verbose']:
                    self.stdout.write('Rebuild LiveBlog {} ({} entries)'.format(liveblog.id,
                                                                                len(entries)))
                    self.stdout.write(url)

                resp = requests.put(url, json=entries)
            else:
                if options['verbose']:
                    self.stdout.write('Delete unpublished LiveBlog {} entries'.format(liveblog.id))
                resp = requests.delete(url)

            resp.raise_for_status()



from rest_framework import routers

from bulbs.liveblog.viewsets import LiveBlogEntryViewSet

api_v1_router = routers.DefaultRouter()
api_v1_router.register(
    r"liveblog/entry",
    LiveBlogEntryViewSet,
    base_name="liveblog-entry"
)

urlpatterns = api_v1_router.urls



from rest_framework import serializers

from djbetty.serializers import ImageFieldSerializer

from bulbs.utils.fields import RichTextField
from bulbs.utils.data_serializers import BaseEntrySerializer, CopySerializer


class GuideToEntrySerializer(BaseEntrySerializer, CopySerializer):
    image = ImageFieldSerializer(required=False, default=None, allow_null=True)


class GuideToChildSerializer(serializers.Serializer):
    entries = GuideToEntrySerializer(many=True, required=False, child_label="entry")


class GuideToParentSerializer(serializers.Serializer):
    copy = RichTextField(required=False, field_size="long")
    sponsor_brand_messaging = serializers.CharField(required=False)
    sponsor_product_shot = ImageFieldSerializer(
        required=False,
        label="Sponsor Product Shot (1x1 Image)"
    )



from rest_framework import serializers

from django.conf import settings

from bulbs.sections.models import Section


class GlanceContentSerializer(serializers.Serializer):

    type = serializers.IntegerField(required=True, allow_null=False)
    id = serializers.CharField()
    title = serializers.CharField()
    url = serializers.URLField()
    modified = serializers.DateTimeField()
    published = serializers.DateTimeField()
    campaign_id = serializers.IntegerField()
    description = serializers.CharField()
    feature_type = serializers.CharField()
    images = serializers.ListField(
        child=serializers.URLField()
    )
    videos = serializers.ListField(
        child=serializers.URLField()
    )
    authors = serializers.ListField(
        child=serializers.DictField()
    )
    sections = serializers.ListField(
        child=serializers.DictField()
    )
    tags = serializers.ListField(
        child=serializers.DictField()
    )

    def to_representation(self, obj):

        # TODO: Move into common code?
        sections = Section.objects.filter(
            pk__in=[s.rpartition('.')[-1] for s in obj.percolate_sections()]
        ).order_by('name').all()

        default_author = getattr(settings, 'GLANCE_FEED_AUTHOR', None)
        if default_author:
            authors = [{
                'id': 0,
                'label': default_author,
            }]
        else:
            authors = [
                {'id': author.id,
                 'label': author.get_full_name()}
                for author in obj.authors.all()
            ]

        # Note: Currently only sets "article" and "video". Type "graphic" not set.
        content_type = 'article'

        # Simple way to look for videos. Does not check for embeds.
        videos = []
        if hasattr(obj, 'videohub_ref'):
            content_type = 'video'
            videos = [obj.videohub_ref.get_hub_url()]

        return {
            'type': content_type,
            'id': obj.id,
            'title': obj.title,
            'modified': obj.last_modified.isoformat(),
            'published': obj.published.isoformat(),
            # Note: Could also check if part of special coverage
            'campaign_id': obj.tunic_campaign_id,
            'description': obj.description,
            'feature_type': (obj.feature_type.name if obj.feature_type else None),
            'images': [
                obj.thumbnail.get_crop_url(ratio='16x9'),
            ],
            'url': self.context['request'].build_absolute_uri(obj.get_absolute_url()),
            'body': getattr(obj, 'body', ''),
            'authors': authors,
            'sections': [
                {'id': section.id,
                 'label': section.name,
                 }
                for section in sections
            ],
            'tags': [
                {'id': tag.id,
                 'label': tag.name,
                 }
                for tag in obj.ordered_tags()
            ],
            'videos': videos,
        }



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('liveblog', '0002_liveblogresponse_internal_name'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='liveblogresponse',
            options={'ordering': ['ordering']},
        ),
    ]
